

/*
 Example sketch for the Xbox Wireless Reciver library - developed by Kristian Lauszus
 It supports up to four controllers wirelessly
 For more information see the blog post: http://blog.tkjelectronics.dk/2012/12/xbox-360-receiver-added-to-the-usb-host-library/ or
 send me an e-mail:  kristianl@tkjelectronics.com
 */

#include <XBOXRECV.h>
#include <Servo.h>


USB Usb;
XBOXRECV Xbox(&Usb);
Servo talonLeft;
Servo talonRight;
boolean isEnabled = false;
boolean isConnected = false;
boolean isConnectedPrior = false;

const AnalogHat leftSide = LeftHatY; // Left|Right Hat X|Y
const AnalogHat rightSide = RightHatY;
const Button throttle = L2; // L2 or R2

int j;
long scaledResult; 

long previousMillis = 0;
long interval = 10;

uint8_t getAnalogHatU8(AnalogHat a, uint8_t controller = 0 ) {
  return (int8_t)highByte( Xbox.getAnalogHat(a, controller) ) + 128;
}
uint16_t getAnalogHatU16(AnalogHat a, uint8_t controller = 0 ) {
  return ( Xbox.getAnalogHat(a, controller) >> 4) + 1024;
}

void Disable(uint8_t pad){
  if( isEnabled ){
    isEnabled=false;
    Xbox.setLedMode(ALTERNATING, pad);
    if ( talonLeft.attached() && talonRight.attached()) {
      talonLeft.write(90);
      talonRight.write(90);
      talonLeft.detach();
      talonRight.detach();
    }
  }
}
void Enable(uint8_t pad){
  if( !isEnabled ){
    isEnabled=true;
    Xbox.setLedOn(LED1, pad);
    if ( !talonLeft.attached() && !talonRight.attached()) {
      talonLeft.attach(2);
      talonRight.attach(3);
    }
  }
}



void DisEnable(uint8_t pad){
  if( isEnabled ){
    Disable(pad);
  }
  else{
    Enable(pad);
  }
}
int autoScale( int originalMin, int originalMax, int newBegin, int
newEnd, int inputValue);


void setup() {
  Serial.begin(115200);
  // while (!Serial); // Wait for serial port to connect - used on Leonardo, 
  // Teensy and other boards with built-in USB CDC serial connection
  if (Usb.Init() == -1) {
    Serial.print(F("\r\nOSC did not start"));
    while (1); //halt
  }
  Serial.print(F("\r\nXbox Wireless Receiver Library Started"));
  Serial.println();
  Enable(1);
}

void loop() {
  Usb.Task();
  if (Xbox.XboxReceiverConnected) {
    uint8_t i = 0; //pad #1
    isConnectedPrior = isConnected;
    isConnected = Xbox.Xbox360Connected[i];
    if( !isConnected && (isConnected  != isConnectedPrior)) {
      Disable(i);
    }
    if (isConnected  && isEnabled) {
      talonLeft.write(autoScale(0, 255, 0, 180, getAnalogHatU8(leftSide, i)));
      talonRight.write(autoScale(0, 255, 0, 180, getAnalogHatU8(rightSide, i)));
    }
    if (Xbox.getButtonClick(XBOX, i)) {
      DisEnable(i);
    }
  }

}


int autoScale( int originalMin, int originalMax, int newBegin, int
newEnd, int inputValue){

  long zeroRefOriginalMax = 0;
  long zeroRefnewEnd = 0;
  long zeroRefCurVal = 0;
  long rangedValue = 0;
  boolean invFlag = 0;

  // Check for out of range inputValues
  if (inputValue < originalMin) {
    inputValue = originalMin;
  }
  if (inputValue > originalMax) {
    inputValue = originalMax;
  }

  // Zero Refference the values
  zeroRefOriginalMax = originalMax - originalMin;

  if (newEnd > newBegin){
    zeroRefnewEnd = newEnd - newBegin;
  }
  else
  {
    zeroRefnewEnd = newBegin - newEnd;
    invFlag = 1;
  }

  zeroRefCurVal = inputValue - originalMin;


  // Check for originalMin > originalMax  - the math for all other cases i.e. negative numbers seems to work out fine
  if (originalMin > originalMax ) {
    return 0;
  }

  if (invFlag == 0){
    rangedValue =  ((zeroRefCurVal * zeroRefnewEnd) /
      zeroRefOriginalMax) + newBegin ;
  }
  else     // invert the ranges
  {  
    rangedValue =  newBegin - ((zeroRefCurVal * zeroRefnewEnd) /
      zeroRefOriginalMax)  ;
  }

  return rangedValue;
}


